#extension GL_OES_EGL_image_external : enable

// This shader is intended to run on a single-pixel framebuffer in order to
// compute an average color over the region defined by uFilterBounds.

// Needs more experimentation, but initial tests indicate that using full color
// saturation works best.
#define ADJUST_COLOR_SATURATION 0

precision mediump float;

uniform samplerExternalOES sTexture;
uniform vec4 uFilterBounds;  // Contains [xMin, xMax, yMin, yMax].
uniform mat4 uPreviewM;
uniform mat4 uTextureM;

// The square root of how many texture samples we use for filtering.
const float kSampleReso = 4.0;

/** Maps normalized canvas coordinates into the external texture space. */
vec2 mapToExternalTexture(float fx, float fy) {
  vec2 view = vec2(fx, fy) * 2.0 - 1.0;
  vec2 deview = vec2(view.x / uPreviewM[0][0], view.y / uPreviewM[1][1]);
  vec2 tex = (uTextureM * vec4((deview + 1.0) * 0.5, 0, 1)).xy;
  return tex;
}

void main() {
  const float kSampleReso = 4.0;
  vec3 color = vec3(0.0);
  vec2 samplePoint;
  vec2 regionMin = clamp(
      mapToExternalTexture(uFilterBounds[0], uFilterBounds[2]), 0.0, 1.0);
  vec2 regionMax = clamp(
      mapToExternalTexture(uFilterBounds[1], uFilterBounds[3]), 0.0, 1.0);
  for (float y = 0.5; y < kSampleReso; y += 1.0) {
    samplePoint.y = mix(regionMin.y, regionMax.y, y / kSampleReso);
    for (float x = 0.5; x < kSampleReso; x += 1.0) {
      samplePoint.x = mix(regionMin.x, regionMax.x, x / kSampleReso);
      color += texture2D(sTexture, samplePoint).rgb;
    }
  }
  color *= 1.0 / (kSampleReso * kSampleReso);

#if ADJUST_COLOR_SATURATION
  const float kSaturation = 0.75;
  const vec3 kLumaWeights = vec3(0.299, 0.587, 0.114);
  vec3 grayscale = vec3(dot(color, kLumaWeights));
  color = mix(grayscale, color, kSaturation);
#endif

  gl_FragColor = vec4(color, 1.0);
}
