#extension GL_OES_EGL_image_external : enable
precision mediump float;

uniform int uWidth;
uniform int uHeight;
uniform samplerExternalOES sTexture;
varying vec2 vTextureCoord;

const vec3 luminanceWeights = vec3(0.299, 0.587, 0.114);

vec4 readTexture(vec2 coord) {
  return texture2D(sTexture, coord);
}

void main() {
  // Pixel unit size in [0, 1.0) coord
  float pixelSizeX = 1.0f / float(4 * uWidth);
  float pixelSizeY = 1.0f / float(4 * uHeight);

  // The first pass:
  // The pixel position of the 4-by-4 block in the input texture.
  vec2 gid = vec2(vTextureCoord);

  // Starting from (0, 0) location
  vec2 gid_0_0 = gid;
  vec2 gid_1_0 = gid_0_0 + vec2(pixelSizeX, 0);
  vec2 gid_0_1 = gid_0_0 + vec2(0, pixelSizeY);
  vec2 gid_1_1 = gid_0_0 + vec2(pixelSizeX, pixelSizeY);
  vec4 avg_0_0 = 0.25 * (readTexture(gid_0_0) + readTexture(gid_1_0) +
                         readTexture(gid_0_1) + readTexture(gid_1_1));

  // Moving to (2, 0) location
  gid_0_0 = gid_0_0 + vec2(2.0*pixelSizeX, 0);
  gid_1_0 = gid_0_0 + vec2(pixelSizeX, 0);
  gid_0_1 = gid_0_0 + vec2(0, pixelSizeY);
  gid_1_1 = gid_0_0 + vec2(pixelSizeX, pixelSizeY);
  vec4 avg_2_0 = 0.25 * (readTexture(gid_0_0) + readTexture(gid_1_0) +
                         readTexture(gid_0_1) + readTexture(gid_1_1));

  // Moving to (0, 2) location
  gid_0_0 = gid_0_0 + vec2(-2.0*pixelSizeX, 2.0*pixelSizeY);
  gid_1_0 = gid_0_0 + vec2(pixelSizeX, 0);
  gid_0_1 = gid_0_0 + vec2(0, pixelSizeY);
  gid_1_1 = gid_0_0 + vec2(pixelSizeX, pixelSizeY);
  vec4 avg_0_2 = 0.25 * (readTexture(gid_0_0) + readTexture(gid_1_0) +
                         readTexture(gid_0_1) + readTexture(gid_1_1));

  // Moving to (2, 2) location
  gid_0_0 = gid_0_0 + vec2(2.0*pixelSizeX, 0);
  gid_1_0 = gid_0_0 + vec2(pixelSizeX, 0);
  gid_0_1 = gid_0_0 + vec2(0, pixelSizeY);
  gid_1_1 = gid_0_0 + vec2(pixelSizeX, pixelSizeY);
  vec4 avg_2_2 = 0.25 * (readTexture(gid_0_0) + readTexture(gid_1_0) +
                         readTexture(gid_0_1) + readTexture(gid_1_1));

  // The second pass:
  vec4 avg = 0.25 * (avg_0_0 + avg_2_0 + avg_0_2 + avg_2_2);

  float luma = dot(avg.rgb, luminanceWeights);
  gl_FragColor = vec4(luma, luma, luma, luma);
}
